;**********************************************************************
;                                                                     *
;    Filename:	    dec-parallel.asm                                  *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:        Peter Jakab <el@jap.hu>                           *
;                   http://jap.hu/electronic/                         *
;**********************************************************************
;NOTES
;encoder v4.2
;
;**********************************************************************
;HISTORY
;
; 050-20091214 16-bit parallel decoder for 16f57
;
;**********************************************************************
	list      p=16f57


#include <p16f5x.inc>
#include <mrxv4.inc>

LATCH_EN EQU 2
RX_IN	EQU 0
VALID EQU 3

	__CONFIG   _CP_OFF & _WDT_ON & _XT_OSC

mainda		UDATA
rcnt		res 1
tmp		res 1

vectors		CODE 0
  		goto    main		; go to beginning of program

prog		CODE 5

name		dt 0x50, "dec-050.asm jap20091214", 0

wd_handler	btfss PORTA, LATCH_EN
		clrwdt
		return

main
		;HARDWARE INIT
		movlw (1<<RX_IN) | (1<<LATCH_EN)
		tris PORTA
		movlw 0
		tris PORTB
		tris PORTC
		movlw 0
		movwf PORTA ; clear VALID bit

		btfsc STATUS, NOT_TO
		goto not_wd_reset
		; wd reset
		btfss PORTA, LATCH_EN
		goto keep_latch
		; clear momentary outputs

not_wd_reset	;
		; clear outputs
		movlw 0
		movwf PORTB
		movwf PORTC
keep_latch	;

		clrwdt ; clears STATUS<NOT_TO>
		movlw (0<<T0CS) | (1<<PSA) | 0x03 ; prescaler to WDT, 1:8
		option ; min timeout = 72 ms, avg = 144 ms

loop		call mrx_receive
		andlw 0xff
		bnz loop
		call mrx_chk_buf
		andlw 0xff
		bnz loop

		; valid reception
		clrwdt
		movf mrx_buffer, W
		xorlw 0xff
		movwf PORTB
		movf mrx_buffer+1, W
		xorlw 0xff
		movwf PORTC
		movlw (1<<VALID)
		movwf PORTA
		goto loop

		end
